package com.sun.xml.internal.ws.spi.db;

import java.security.PrivilegedExceptionAction;
import java.security.PrivilegedActionException;
import java.security.AccessController;
import java.lang.reflect.Method;

public class MethodGetter extends PropertyGetterBase
{
    private Method method;
    
    public MethodGetter(Method m) {
        this.method = m;
        this.type = m.getReturnType();
    }
    
    public Method getMethod() {
        return this.method;
    }
    
    @Override
    public <A> A getAnnotation(Class<A> annotationType) {
        Class<A> c = annotationType;
        return this.method.getAnnotation(c);
    }
    
    @Override
    public Object get(Object instance) {
        Object[] args = new Object[0];
        try {
            if (this.method.isAccessible()) {
                return this.method.invoke(instance, args);
            }
            MethodGetter.PrivilegedGetter privilegedGetter = new MethodGetter.PrivilegedGetter(this.method, instance);
            try {
                AccessController.doPrivileged(privilegedGetter);
            }
            catch (PrivilegedActionException e) {
                e.printStackTrace();
            }
            return privilegedGetter.value;
        }
        catch (Exception e2) {
            e2.printStackTrace();
            return null;
        }
    }
    
    static class PrivilegedGetter implements PrivilegedExceptionAction
    {
        private Object value;
        private Method method;
        private Object instance;
        
        public PrivilegedGetter(Method m, Object instance) {
            this.method = m;
            this.instance = instance;
        }
        
        @Override
        public Object run() throws IllegalAccessException {
            if (!this.method.isAccessible()) {
                this.method.setAccessible(true);
            }
            try {
                this.value = this.method.invoke(this.instance, new Object[0]);
            }
            catch (Exception e) {
                e.printStackTrace();
            }
            return null;
        }
    }
}
